// An assimilator for SixTrack.
// Based on the sample assimilator supplied with BOINC.

#include <cstdio>

#include "boinc_db.h"
#include "sched_msgs.h"
#include "sched_util.h"
#include "parse.h"
#include "assimilate_handler.h"

using std::vector;

void assimilate_handler(
	WORKUNIT& wu, vector<RESULT>& results, RESULT& canonical_result, char destdir[], char sourcedir[], char klog[], char
	username[], char passwdfile[]
) {
    char outputfile[100];
    char commandline[200];

    int retval;

    SCOPE_MSG_LOG scope_messages(log_messages, SCHED_MSG_LOG::NORMAL);
    scope_messages.printf("[%s] Assimilating\n", wu.name);


    if (wu.canonical_resultid) {
        scope_messages.printf("[%s] Found canonical result - Using Sixtrack Assim\n", wu.name);
        if (! parse_str(canonical_result.xml_doc_out, "<name>", outputfile, 100)){
                log_messages.printf(SCHED_MSG_LOG::CRITICAL, "[%s] Error: couldn't parse xml_doc_out\n", wu.name);
                return;
        }

        if (!strcmp(klog,"true")) {
                strcpy(commandline, "/boinc/projects/lhcathome/bin/klogscript ");
                strcat(commandline, username);
                strcat(commandline, " ");
                strcat(commandline, passwdfile);
                log_messages.printf(SCHED_MSG_LOG::DEBUG, "[%s] Executing command: '%s'\n", wu.name, commandline);
                retval = system(commandline);
                if (retval){
                        log_messages.printf(SCHED_MSG_LOG::CRITICAL, "[%s] klogging failed!\n", wu.name);
                        return;
                }
        }

        strcpy(commandline, "cp ");
        strcat(commandline, sourcedir);
	strcat(commandline, outputfile);
        strcat(commandline, " ");
        strcat(commandline, destdir);
        strcat(commandline, wu.name);
        log_messages.printf(SCHED_MSG_LOG::DEBUG, "[%s] Executing command: '%s'\n", wu.name, commandline);
        retval = system(commandline);
        if (retval){
                log_messages.printf(SCHED_MSG_LOG::CRITICAL, "[%s] Copying failed.\n", wu.name);
                return;
        }   
    } else {
        scope_messages.printf("[%s] No canonical result\n", wu.name);
    }
    if (wu.error_mask&WU_ERROR_COULDNT_SEND_RESULT) {
        log_messages.printf(SCHED_MSG_LOG::CRITICAL, "[%s] Error: couldn't send a result\n", wu.name);
    }
    if (wu.error_mask&WU_ERROR_TOO_MANY_ERROR_RESULTS) {
        log_messages.printf(SCHED_MSG_LOG::CRITICAL, "[%s] Error: too many error results\n", wu.name);
    }
    if (wu.error_mask&WU_ERROR_TOO_MANY_TOTAL_RESULTS) {
        log_messages.printf(SCHED_MSG_LOG::CRITICAL, "[%s] Error: too many total results\n", wu.name);
    }
    if (wu.error_mask&WU_ERROR_TOO_MANY_SUCCESS_RESULTS) {
        log_messages.printf(SCHED_MSG_LOG::CRITICAL, "[%s] Error: too many success results\n", wu.name);
    }
}
